///////////////////////////////////////////////////////////
// POPUPMNU.CPP: Demonstrates creating and responding to a
//               floating pop-up menu.
///////////////////////////////////////////////////////////

#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\menu.h>
#include <stdio.h>
#pragma hdrstop
#include "popupmnu.rc"

//////////////////////////////////
// The main window class.
//////////////////////////////////
class TWndw : public TFrameWindow
{
public:
	TWndw(TWindow *parent, const char far *title);
	void SetupWindow();
	void CleanupWindow();
protected:
	void EvRButtonDown(UINT, TPoint &point);
	void CmCommand1() {MessageBox("CmCommand1 received", "Message");}
	void CmCommand2() {MessageBox("CmCommand2 received", "Message");}
	void CmCommand3() {MessageBox("CmCommand3 received", "Message");}
	void CmPopupItem() {MessageBox("CmPopupItem received", "Message");}
	void CmAddItem();
	void CmDeleteItem();
	void CmEnableAddItem(TCommandEnabler &commandEnabler);
	void CmEnableDeleteItem(TCommandEnabler &commandEnabler);

	TMenu *popupMenu;
DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1(TWndw, TFrameWindow)
	EV_WM_RBUTTONDOWN,
	EV_COMMAND(CM_COMMAND1, CmCommand1),
	EV_COMMAND(CM_COMMAND2, CmCommand2),
	EV_COMMAND(CM_COMMAND3, CmCommand3),
	EV_COMMAND(CM_POPUPITEM, CmPopupItem),
	EV_COMMAND(CM_ADDITEM, CmAddItem),
	EV_COMMAND_ENABLE(CM_ADDITEM, CmEnableAddItem),
	EV_COMMAND(CM_DELETEITEM, CmDeleteItem),
	EV_COMMAND_ENABLE(CM_DELETEITEM, CmEnableDeleteItem),
END_RESPONSE_TABLE;

///////////////////////////////////////////////////////////
// TWndw::TWndw()
///////////////////////////////////////////////////////////
TWndw::TWndw(TWindow *parent, const char far *title):
		TFrameWindow(parent, title)
{
	// Add the menu to the main window.
	AssignMenu(MENU_1);

	// Position and size the main window.
	Attr.X = 50;
	Attr.Y = 50;
	Attr.W = GetSystemMetrics(SM_CXSCREEN) / 1.5;
	Attr.H = GetSystemMetrics(SM_CYSCREEN) / 1.5;
}

///////////////////////////////////////////////////////////
// TWndw::SetupWindow()
///////////////////////////////////////////////////////////
void TWndw::SetupWindow()
{
	TFrameWindow::SetupWindow();

	HINSTANCE hInstance = GetApplicationObject()->GetInstance();
	popupMenu = new TMenu(hInstance, MENU_2);
}

///////////////////////////////////////////////////////////
// TWndw::CleanupWindow()
///////////////////////////////////////////////////////////
void TWndw::CleanupWindow()
{
	delete popupMenu;
}


///////////////////////////////////////////////////////////
// TWndw::EvRButtonDown()
///////////////////////////////////////////////////////////
void TWndw::EvRButtonDown(UINT, TPoint &point)
{
	// Get the Test menu's handle.
	HMENU hMenu = popupMenu->GetSubMenu(0);

	// Get the position of the window.
	TRect rect;
	GetWindowRect(rect);

	// Create a TPopupMenu object for the Test menu.
	TPopupMenu popupMenu(hMenu);

	// Display the floating menu.
	popupMenu.TrackPopupMenu(TPM_LEFTALIGN,	point.x + rect.left,
		point.y + rect.top + 50, 0, HWindow, 0);
}

///////////////////////////////////////////////////////////
// TWndw::CmAddItem()
///////////////////////////////////////////////////////////
void TWndw::CmAddItem()
{
	// Get the Test menu's handle.
	HMENU hMenu = popupMenu->GetSubMenu(0);

	// Create a TMenu object for the Test menu.
	TMenu sMenu(hMenu);

	// Get the number of items in the Test menu.
	int count = sMenu.GetMenuItemCount();

	// If there are less than 4 items in the menu...
	if (count < 7) {
		// Add the new item.
		char buffer[15];
		sprintf(buffer, "Command &%d", count - 3);
		sMenu.AppendMenu(MF_STRING, CM_COMMAND1 + count - 4, buffer);
	}
}

///////////////////////////////////////////////////////////
// TWndw::CmDeleteItem()
///////////////////////////////////////////////////////////
void TWndw::CmDeleteItem()
{
	// Get the Test menu's handle.
	HMENU hMenu = popupMenu->GetSubMenu(0);

	// Create a TMenu object for the Test menu.
	TMenu sMenu(hMenu);

	// Get the number of items in the Test menu.
	int count = sMenu.GetMenuItemCount();

	// If there are new items in the menu...
	if (count > 4)
		// Delete a new item.
		sMenu.DeleteMenu(count - 1, MF_BYPOSITION);
}

///////////////////////////////////////////////////////////
// TWndw::CmEnableAddItem(TCommandEnabler &commandEnabler)
///////////////////////////////////////////////////////////
void TWndw::CmEnableAddItem(TCommandEnabler &commandEnabler)
{
	HMENU hMenu = popupMenu->GetSubMenu(0);
	TMenu sMenu(hMenu);
	int count = sMenu.GetMenuItemCount();
	commandEnabler.Enable(count < 7);
}

///////////////////////////////////////////////////////////
// TWndw::CmEnableDeleteItem(TCommandEnabler &commandEnabler)
///////////////////////////////////////////////////////////
void TWndw::CmEnableDeleteItem(TCommandEnabler &commandEnabler)
{
	HMENU hMenu = popupMenu->GetSubMenu(0);
	TMenu sMenu(hMenu);
	int count = sMenu.GetMenuItemCount();
	commandEnabler.Enable(count != 4);
}

//////////////////////////////////
// The application class.
//////////////////////////////////
class TApp : public TApplication
{
public:
	TApp(): TApplication() {}
	void InitMainWindow();
};

///////////////////////////////////////////////////////////
// TApp::InitMainWindow()
///////////////////////////////////////////////////////////
void TApp::InitMainWindow()
{
	TFrameWindow *wndw = new TWndw(0, "Popup Menu Item Add/Delete");
	SetMainWindow(wndw);
}

///////////////////////////////////////////////////////////
// OwlMain()
///////////////////////////////////////////////////////////
int OwlMain(int, char*[])
{
	return TApp().Run();
}
